package com.fossgalaxy.games.tbs.actions;

import com.fossgalaxy.games.tbs.GameState;
import com.fossgalaxy.games.tbs.entity.Entity;
import com.fossgalaxy.games.tbs.entity.Resource;
import com.fossgalaxy.games.tbs.parameters.ResourceType;
import org.codetome.hexameter.core.api.CubeCoordinate;

import java.awt.*;
import java.util.Objects;

/**
 * Created by pwillic on 18/05/2018.
 */
public abstract class AbstractMineAction extends AbstractAction {

    protected final ResourceType resourceType;

    public AbstractMineAction(ResourceType resourceType) {
        this.resourceType = Objects.requireNonNull(resourceType);
    }

    @Override
    public boolean isPossible(Entity entity, GameState s, CubeCoordinate co) {
        if (!super.isPossible(entity, s, co)) return false;

        Resource resource = s.getResourceAt(co);
        if (resource == null) return false;
        if (!resource.getType().equals(resourceType)) return false;
        if (resource.getAmountPerTurn() <= 0) return false;
        return true;
    }

    @Override
    public boolean canAutomate() {
        return true;
    }

    @Override
    public Color getHintColour() {
        return HINT_MINE_COLOUR;
    }

    @Override
    public Color getBorderColour() {
        return HINT_MINE_COLOUR;
    }

    @Override
    public String toString() {
        return "Mine: " + resourceType.getName();
    }

    @Override
    public String getCategory() {
        return AbstractAction.CAT_PRODUCTION;
    }
}
