package com.fossgalaxy.games.tbs.ai.rules.benchmark;

import com.fossgalaxy.games.tbs.App;
import com.fossgalaxy.games.tbs.GameDef;
import com.fossgalaxy.games.tbs.GameState;
import com.fossgalaxy.games.tbs.ai.AIFactory;
import com.fossgalaxy.games.tbs.ai.Controller;
import com.fossgalaxy.games.tbs.io.SettingsIO;
import com.fossgalaxy.games.tbs.io.map2.MapDef;
import com.fossgalaxy.games.tbs.order.Order;
import com.fossgalaxy.games.tbs.order.OrderProcessor;
import com.fossgalaxy.games.tbs.parameters.GameSettings;
import com.fossgalaxy.games.tbs.parameters.ResourceType;
import com.fossgalaxy.games.tbs.rules.Rule;
import org.openjdk.jmh.annotations.*;
import org.openjdk.jmh.runner.Runner;
import org.openjdk.jmh.runner.RunnerException;
import org.openjdk.jmh.runner.options.Options;
import org.openjdk.jmh.runner.options.OptionsBuilder;

import java.util.*;
import java.util.concurrent.TimeUnit;

@State(Scope.Thread)
public class ControllerBenchmark {


    @Param({"random"})
    String player;

    @Param({"game.json"})
    String game;
    @Param({"map.json"})
    String level;

    @Param({"com.fossgalaxy.game"})
    String gamePackage;


    @Benchmark
    @BenchmarkMode(Mode.Throughput)
    @OutputTimeUnit(TimeUnit.MINUTES)
    public void testMethod() {
        SettingsIO io = SettingsIO.buildWithExtras(gamePackage);
        GameDef gameDef = io.loadGameDef(game);
        GameSettings common = io.loadSettings(gameDef);
        common.finish(io);
        AIFactory factory = new AIFactory(io, gameDef.getEvalFileName(), gameDef.getRuleFileName(), gameDef.getAiFileName());


        //load map
        MapDef map = io.loadMapDef(level);

        System.out.println(map);
        GameState level = map.buildState(common);


        // HACK TODO FIX THIS
        for (ResourceType type : common.getResouceTypes()) {
            for (int i = 0; i < level.getNumberOfPlayers(); i++) {
                level.addResource(i, type, map.getStartingResource(i, type.getName()));
            }
        }

        int numPlayers = level.getNumberOfPlayers();

        List<GameState> playerStates = new ArrayList<>(numPlayers);
        for (int i = 0; i < numPlayers; i++) {
            playerStates.add(new GameState(level));
        }
        String[] agents = new String[numPlayers];
        Arrays.fill(agents, player);
        Controller[] controllers = new Controller[numPlayers];
        for (int i = 0; i < numPlayers; i++) {
            controllers[i] = factory.buildAI(agents[i], common);
        }

        //create the thing that's doing the turn tracking.
        OrderProcessor processor = new OrderProcessor(level, numPlayers);

        for (Rule rule : common.getVictoryConditions()) {
            processor.addRule(rule);
        }

        //TODO this might be better inside the turn processor?
        for (int i = 0; i < gameDef.getTurnLimit(); i++) {
            processor.setupTurn();

            //figure out who's turn it is.
            int playerID = processor.getCurrentPlayer();
            Controller player = controllers[playerID];

            //process the controller's orders
            GameState curr = new GameState(level);
            Map<UUID, Order> orders = player.doTurn(playerID, curr);

            processor.doOrderBulk(orders);
            processor.finishTurn();

            if (!Objects.equals(processor.getWinner(), Rule.NO_WINNER)) {
                break;
            }
        }

    }

    public static void main(String[] args) throws RunnerException {
        String game = args[0];
        String map = args[1];
        String gamePackage = args[2];
        String[] controllers = Arrays.stream(args).skip(3).toArray(String[]::new);

        Options opt = new OptionsBuilder()
                .include(ControllerBenchmark.class.getSimpleName())
                .param("gamePackage", gamePackage)
                .param("game", game)
                .param("level", map)
                .param("player", controllers)
                .forks(1)
                .build();

        new Runner(opt).run();
    }
}
